/*
 * @(#)SadissComboBoxEditor.java 1.0 10/24/2002
 * Copyright 2002 Senamirmir Project
 *
 * You may distribute and/or modify Sadiss under the conditions 
 * of the LaTeX Project Public License. The license is located at
 * http://www.latex-project.org/lppl.txt
 */

package keffa ;

import java.io.* ;
import java.awt.* ;
import javax.swing.* ;
import java.awt.geom.* ;
import javax.swing.text.* ;
import java.awt.event.* ;
import javax.swing.plaf.metal.*;
import javax.swing.plaf.*;
import java.util.* ;

/**
 * <!-- ----------------------------------------------------------------------- -->
 * Sadiss is a Unicode text editor for Ethiopic. It supports multiple 
 * documents editing, UTF-8, Unicode text search, and printing.  A Unicode 
 * characters may be entered using their code numbers. For instance, \\u1200 
 * is the first character of Ethiopic, \\u1201 the second and so on.
 *
 * <p>
 *    Sadiss is tailored for Ethiopic. It supports the modified KWK keyboard 
 *    layout, provides Amharic locale, and Amharic help.
 * </p>
 *
 * <p>
 *    The Sadiss class is a subclass of JFrame, and sets up the desktop, the
 *    initial document frame, and the user interface. 
 * </p>
 *       
 * @author abass alamene
 * @version 1.0 10/24/2002
 * <!-- ----------------------------------------------------------------------- -->
 */

public class Sadiss extends JFrame {
   JDesktopPane desktop = new JDesktopPane() ;   

   /**
    * Sadiss constructor sets up the frame, desktop, internalframe, toolbar, 
    * and emnubar
    */
   public Sadiss() {
      super("Sadiss 1.0") ;
      setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE) ;
      int inset = 30 ;

      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	   setBounds ( 0, 0, screenSize.width, screenSize.height - inset );

         // setup desktop
      desktop.putClientProperty("JDesktopPane.dragMode", "outline");
      Dimension d = getSize() ;
      desktop.setPreferredSize(d);
      SadissJournal.setDesktop(desktop) ;

         // implements a window listener via WindowAdapter
      addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent event) {
            JDesktopPane desktop  = SadissJournal.getDesktop() ;
            JInternalFrame[] sdf = (JInternalFrame[]) desktop.getAllFrames() ;
            try {
               for (int i=0; i < sdf.length; i++) {
                  desktop.setSelectedFrame(sdf[i]) ; 
                  sdf[i].setClosed(true) ;
               }

               sdf = (JInternalFrame[]) desktop.getAllFrames() ;
               if (sdf.length == 0) 
                  System.exit(0) ;
            } catch (Exception ex) {
               System.out.println(ex) ;
            }
         }
      } ) ;

         // instantiate an object of SadissControl
      SadissControl sc = new SadissControl(this) ;

         // log the current SadissControl in SadissJournal
      SadissJournal.setSadissControl(sc) ;

         // add a menu bar 
      setJMenuBar(sc.getSadissMenuBar()) ;

         // add a toolbar
      getContentPane().add(sc.getToolBar(), BorderLayout.NORTH) ;     

         // create the initial SadissDocumentFrame
      JInternalFrame jf = sc.createDocumentFrame("untitle1.utf") ;
         
         // add the SadissDocumentFrame into desktop, the desktop into JFrame
      desktop.add(jf) ;
      getContentPane().add(desktop, BorderLayout.CENTER) ;

         // show the frame
      setVisible(true) ;

         // make this initila internal frame active or focussed
      try {
         jf.setSelected(true) ;
      } catch (Exception ex) {
         System.out.println(ex) ;
      }
   }

   /**
    * An entry point to Sadiss execution
    * @param args captures the command line arguments if there are any
    */
   public static void main(String[] args) {

      String language;
      String country;

      if (args.length != 2) {
          language = new String("am");
          country = new String("ET");
      } else {
          language = new String(args[0]);
          country = new String(args[1]);
      }

         // set the current locale
      Locale currentLocale = new Locale(language, country) ;
      SadissJournal.setLocale(currentLocale) ;
      ResourceBundle rb = 
               ResourceBundle.getBundle("keffa.SadissResourceBundle", currentLocale) ;
      SadissJournal.setResourceBundle(rb) ;

         // set default font
      SadissJournal.createFont(rb.getString("defaultfontfilename"),  16) ;         
                               
         // set LookAndFeel
      JDialog.setDefaultLookAndFeelDecorated(true);
      JFrame.setDefaultLookAndFeelDecorated(true);
      Toolkit.getDefaultToolkit().setDynamicLayout(true);
      System.setProperty("sun.awt.noerasebackground","true");

      try {
         MetalLookAndFeel.setCurrentTheme(new LowContrastMetalTheme()) ;
         UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
         UIManager.put("ToolBar.height", new Integer(15)) ;
      } catch (Exception e) { 
         System.out.println(e) ; 
      }

      new Sadiss() ;
   }
}