/*
 * @(#)SadissControl.java 1.0 10/24/2002
 * Copyright 2002 Senamirmir Project
 *
 * You may distribute and/or modify Sadiss under the conditions 
 * of the LaTeX Project Public License. The license is located at
 * http://www.latex-project.org/lppl.txt
 */


package keffa ;

import java.io.* ;
import java.awt.* ;
import java.awt.geom.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.border.* ;
import javax.swing.event.* ;
import javax.swing.undo.* ;
import javax.swing.text.* ;
import java.net.URL ;
import java.awt.print.*;
import java.beans.* ;
import javax.swing.plaf.metal.*;
import javax.swing.plaf.*;
import javax.print.attribute.PrintRequestAttributeSet ;
import javax.print.attribute.HashPrintRequestAttributeSet ;

/**
 * <!-- ----------------------------------------------------------------------- -->
 * SadissControl builds and manages Sadiss' user interface. It builds toolbar,
 * menu bar, document frames, and handles their events. Most common events are 
 * handled using the Action interface. 
 * @author abass alamene
 * @version 10/24/2002
 * <!-- ----------------------------------------------------------------------- -->
 */
public class SadissControl {

   private SadissToolBar sadissToolBar  ;
   private SadissMenuBar sadissMenuBar ;
	private final String[] SadissX = {"utf", "txt", "java", "htm", "html", "tex"} ;
   private final String desc = "*.utf, *.txt, *.java, *.htm, *.html, *.tex" ;

   private static int xDocPos = 0 ;
   private static int yDocPos = 0 ;
   private static int inset = 20;
   private static int internalFrameCount = 1;
   private Component owner ;
   private StyledDocument doc ;
   private SadissTextPane textPane = new SadissTextPane() ;
   private static boolean firstSave = true ;
   private static String currentFileName = "" ; 
   private static File currentFile ; 
   private Hashtable actionHashtable = new Hashtable() ;  
   private String imagePath = "keffa/icons/" ; 
   private SadissFileService sfs ;
   protected UndoManager undoManager = new UndoManager() ;
   private static Font font = null ;

   private UndoAction undoAction               = new UndoAction() ;
   private RedoAction redoAction               = new RedoAction() ;
   private NewAction newAction                 = new NewAction() ;
   private SaveAction saveAction               = new SaveAction() ;
   private SaveAsAction saveAsAction           = new SaveAsAction() ;
   private OpenAction openAction               = new OpenAction() ;
   private PrintAction printAction             = new PrintAction() ;
   private SearchAction searchAction           = new SearchAction() ;
   private CloseAction closeAction             = new CloseAction() ;
   private ExitAction exitAction               = new ExitAction() ;
   private HelpAction helpAction               = new HelpAction() ;
   private ThemeAction themeAction             = new ThemeAction() ;
   private UnicodeAction unicodeAction         = new UnicodeAction() ;

   private Action copyAction ;
   private Action cutAction ;
   private Action pasteAction ;
   private Action selectAllAction ;

   /**
    * Empty Constructor
    */
   SadissControl() {}

   
   /**
    * Constructor sets filters, undo manager, ...
    * @param owner current object
    */
   SadissControl(Component owner) {
      this.owner = owner ;

      hashDefaultActions() ;

    	SadissFileFilter sff = new SadissFileFilter(SadissX, desc) ;
      sfs = SadissFileService.getInstance(owner, sff) ;
      font = SadissJournal.getDefaultFont() ;     
      sadissToolBar = new SadissToolBar() ;  // must instantiate before SadissMenuBar
//      sadissMenuBar = new SadissMenuBar() ;  // always place this after the previous statement
   }

   /**
    * Returns SadissDocumentFrame, which is an JInternalFrame type
    * @param fileName name of the document
    */
   public SadissDocumentFrame createDocumentFrame(String fileName) {
      return new SadissDocumentFrame(fileName) ;
   }

   /**
    * Returns an initialized instance of ThemeAction
    */
   public Action getUnicodeAction() {
      unicodeAction.putValue(Action.SHORT_DESCRIPTION, "Unicode") ; 
      String path = imagePath + "Unicode16.gif" ;
      URL url = ClassLoader.getSystemResource(path);   
      unicodeAction.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;
      return unicodeAction ; 
   } 

   /**
    * Returns an initialized instance of ThemeAction
    */
   public Action getThemeAction() {
      themeAction.putValue(Action.SHORT_DESCRIPTION, "Theme") ; 
      return themeAction ; 
   } 

   /**
    * Returns an initialized instance of SearchAction
    */
   public Action getSearchAction() {
      searchAction.putValue(Action.SHORT_DESCRIPTION, "Search") ; 
      String path = imagePath + "Find16.gif" ;
      URL url = ClassLoader.getSystemResource(path);   
      searchAction.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;
      return searchAction ; 
   } 

   /**
    * Returns an initialized instance of NewAction
    */
   public Action getNewAction() {
      newAction.putValue(Action.SHORT_DESCRIPTION, "New") ; 
      String path = imagePath + "New16.gif" ;
      URL url = ClassLoader.getSystemResource(path);  
      newAction.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;
      return newAction ; 
   } 

   /**
    * Returns an initialized instance of CloseAction
    */
   public Action getCloseAction() {
      closeAction.putValue(Action.SHORT_DESCRIPTION, "Close") ; 
      return closeAction ; 
   } 

   /**
    * Returns an initialized instance of SaveAction
    */
   public Action getSaveAction() { 
      saveAction.putValue(Action.SHORT_DESCRIPTION, "Save") ; 
      String path = imagePath + "Save16.gif" ;
      URL url = ClassLoader.getSystemResource(path);   
      saveAction.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;
      return saveAction ; 
   } 

   /**
    * Returns an initialized instance of SaveAsAction
    */
   public Action getSaveAsAction() {
      saveAsAction.putValue(Action.SHORT_DESCRIPTION, "SaveAs") ; 
      String path = imagePath + "SaveAs16.gif" ;
      URL url = ClassLoader.getSystemResource(path);   
      saveAsAction.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;
      return saveAsAction ; 
   } 

   /**
    * Returns an initialized instance of OpenAction
    */
   public Action getOpenAction() { 
      openAction.putValue(Action.SHORT_DESCRIPTION, "Open") ; 
      String path = imagePath + "Open16.gif" ;
      URL url = ClassLoader.getSystemResource(path);   
      openAction.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;
      return openAction ; 
   } 

   /**
    * Returns an initialized instance of PrintAction
    */
   public Action getPrintAction() { 
      printAction.putValue(Action.SHORT_DESCRIPTION, "Print") ; 
      String path = imagePath + "Print16.gif" ;
      URL url = ClassLoader.getSystemResource(path);   
      printAction.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;

      return printAction ; 
   } 

   /**
    * Returns an initialized instance of ExitAction
    */
   public Action getExitAction() {
      exitAction.putValue(Action.SHORT_DESCRIPTION, "Exit") ; 
      int shortCut = 'X' ;
      exitAction.putValue(Action.MNEMONIC_KEY, new Integer(shortCut) ) ; 
      return exitAction ; 
   } 

   /**
    * Returns an initialized instance of UndoAction
    */
   public Action getUndoAction() {
      undoAction.putValue(Action.SHORT_DESCRIPTION, "Undo") ; 
      int shortCut = 'U' ;
      undoAction.putValue(Action.MNEMONIC_KEY, new Integer(shortCut) ) ; 
      return undoAction ; 
   } 

   /**
    * Returns an initialized instance of RedoAction
    */
   public Action getRedoAction() {
      redoAction.putValue(Action.SHORT_DESCRIPTION, "Redo") ; 
      int shortCut = 'R' ;
      redoAction.putValue(Action.MNEMONIC_KEY, new Integer(shortCut) ) ; 
      return redoAction ; 
   } 

   /**
    * Returns an initialized instance of ExitAction
    */
   public Action getHelpAction() {
      helpAction.putValue(Action.SHORT_DESCRIPTION, "Help") ; 
      String path = imagePath + "Help16.gif" ;
      URL url = ClassLoader.getSystemResource(path);   
      helpAction.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;

      return helpAction ; 
   } 

   /**
    * Returns an instance of a menubar
    */
   public JMenuBar getSadissMenuBar() {
      if (sadissMenuBar == null) {         
         sadissMenuBar = new SadissMenuBar() ;
         sadissMenuBar.setEnabled(false) ;
      }
      return sadissMenuBar.makeMenuBar() ;
   }


   /**
    * <!-- ----------------------------------------------------------------------- -->
    * SadissMenuBar is an inner class that builds menubar and popup menu for 
    * Sadiss
    * <!-- ----------------------------------------------------------------------- -->
    */
   class SadissMenuBar {
      JMenuBar menuBar ; 
      SadissPopupMenu popupMenu ;

      /**
       * Returns a menubar 
       */
      public SadissPopupMenu getPopupMenu() {
         return popupMenu ;
      }

      /**
       * Returns a menubar 
       */
      public JMenuBar makeMenuBar() {
         return menuBar ;
      }

      /**
       * Disables selected toolbar buttons when there isn't object to work on
       */
      void setEnabled(boolean state) {
         popupMenu.setEnabled(state) ;
      }
   
      /**
       * Constructor
       */
      SadissMenuBar() {
         ResourceBundle rb = SadissJournal.getResourceBundle() ;
         popupMenu = new SadissPopupMenu() ;
         menuBar = new JMenuBar() ;

             // setting the theme menu
         JMenu theme = new SadissMenu("Theme") ;

         theme.setText(rb.getString("theme")) ;
         theme.setMnemonic('T') ;

         JMenuItem menuItem = new SadissMenuItem( getThemeAction() ) ;
         menuItem.setText(rb.getString("khakitheme")) ; 
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('S') ;
         menuItem.setActionCommand("0") ;
         theme.add(menuItem) ;

         menuItem = new SadissMenuItem( getThemeAction() ) ;
         menuItem.setText(rb.getString("steelbluetheme")) ; 
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('B') ;
         menuItem.setActionCommand("1") ;
         theme.add(menuItem) ;

         menuItem = new SadissMenuItem( getThemeAction() ) ;
         menuItem.setText(rb.getString("feldspartheme")) ; 
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('F') ;
         menuItem.setActionCommand("2") ;
         theme.add(menuItem) ;

         menuItem = new SadissMenuItem( getThemeAction() ) ;
         menuItem.setText(rb.getString("lightcharcoaltheme")) ; 
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('D') ;
         menuItem.setActionCommand("3") ;
         theme.add(menuItem) ;

         menuItem = new SadissMenuItem( getThemeAction() ) ;
         menuItem.setText(rb.getString("highcontrasttheme")) ; 
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('L') ;
         menuItem.setActionCommand("4") ;
         theme.add(menuItem) ;

         menuItem = new SadissMenuItem( getThemeAction() ) ;
         menuItem.setText(rb.getString("lowcontrasttheme")) ; 
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('G') ;
         menuItem.setActionCommand("5") ;
         theme.add(menuItem) ;

         menuItem = new SadissMenuItem( getThemeAction() ) ;
         menuItem.setText(rb.getString("defaultmetal")) ; 
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('D') ;
         menuItem.setActionCommand("6") ;
         theme.add(menuItem) ;

             // setting the window menu
         JMenu window = new SadissMenu("Window") ;      
         window.setText(rb.getString("window")) ;
         window.setMnemonic('W') ;

             // setting the help menu
         JMenu help = new SadissMenu("Help") ;      
         help.setText(rb.getString("help")) ;
         help.setMnemonic('H') ;

         menuItem = new SadissMenuItem( getHelpAction() ) ;
         menuItem.setText(rb.getString("help")) ; 
         menuItem.setIcon(null) ;
         help.add(menuItem) ;

             // setting the file menu
         JMenu file = new SadissMenu("File") ;
         file.setText(rb.getString("file")) ;
         file.setMnemonic('F') ;

         menuItem = new SadissMenuItem( getNewAction() ) ;
         menuItem.setText(rb.getString("new")) ; 
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('N') ;
         file.add(menuItem) ;

         menuItem = new SadissMenuItem(getOpenAction()) ;
         menuItem.setText(rb.getString("open")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('O') ;
         file.add(menuItem) ;

         menuItem = new SadissMenuItem(getCloseAction()) ;
         menuItem.setText(rb.getString("close")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('C') ;
         file.add(menuItem) ;

         file.addSeparator() ;

         menuItem = new SadissMenuItem(getSaveAction()) ;
         menuItem.setText(rb.getString("save")) ;
         menuItem.setMnemonic('S') ;
         menuItem.setIcon(null) ;
         file.add(menuItem) ;

         menuItem = new SadissMenuItem(getSaveAsAction()) ;
         menuItem.setText(rb.getString("saveas")) ;
         menuItem.setMnemonic('A') ;
         menuItem.setIcon(null) ;
         file.add(menuItem) ;

         file.addSeparator() ;

         menuItem = new SadissMenuItem(getPrintAction()) ;
         menuItem.setText(rb.getString("print")) ;
         menuItem.setMnemonic('P') ;
         menuItem.setIcon(null) ;
         file.add(menuItem) ;

         file.addSeparator() ;

         menuItem = new SadissMenuItem(getExitAction()) ;
         menuItem.setText(rb.getString("exit")) ;
         menuItem.setMnemonic('X') ;
         menuItem.setIcon(null) ;
         file.add(menuItem) ;

            // setting the edit menu
         JMenu edit = new SadissMenu("Edit") ;      
         edit.setText(rb.getString("edit")) ;

         menuItem = new SadissMenuItem(getUndoAction()) ;
         menuItem.setText(rb.getString("undo")) ;
         menuItem.setMnemonic('U') ;
         menuItem.setIcon(null) ;
         edit.add(menuItem) ;

         menuItem = new SadissMenuItem(getRedoAction()) ;
         menuItem.setText(rb.getString("redo")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('R') ;
         edit.add(menuItem) ;

         edit.addSeparator() ;

         menuItem = new SadissMenuItem(copyAction) ;
         menuItem.setText(rb.getString("copy")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('C') ;
         edit.add(menuItem) ;

         menuItem = new SadissMenuItem(cutAction) ;
         menuItem.setText(rb.getString("cut")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('T') ;
         edit.add(menuItem) ;

         menuItem = new SadissMenuItem(pasteAction) ;
         menuItem.setText(rb.getString("paste")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('P') ;
         edit.add(menuItem) ;

         menuItem = new SadissMenuItem(selectAllAction) ;
         menuItem.setText(rb.getString("selectall")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('L') ;
         edit.add(menuItem) ;

         edit.addSeparator() ;
         menuItem = new SadissMenuItem(getUnicodeAction()) ;
         menuItem.setText(rb.getString("unicode")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('U') ;
         edit.add(menuItem) ;
           
         menuItem = new SadissMenuItem(getSearchAction()) ;
         menuItem.setText(rb.getString("search")) ;
         menuItem.setIcon(null) ;
         menuItem.setMnemonic('S') ;
         edit.add(menuItem) ;
         edit.setMnemonic('E') ;

         menuBar.add(file) ;
         menuBar.add(edit) ;
         menuBar.add(theme) ;
         menuBar.add(window) ;
         menuBar.add(help) ;
      }  // close constructor
   }  // close class


   /**
    * Builds a combobox for font size
    */
   public JComboBox createComboBox(String[] items, int size) {
      JComboBox jcb = new JComboBox(items) ;
      jcb.setEditable(true) ;
      jcb.setMaximumRowCount(20) ;
      jcb.setEditor(new SadissComboBoxEditor(items[0], size)) ;
      return jcb ;
   }

   /**
    * builds hashtable of the JTextPane actions
    */
   public void hashDefaultActions() {
      JTextPane textPane = new JTextPane() ;
      Action[] actions = textPane.getActions() ;
      for (int i=0; i < actions.length; i++) {
         String name = (String) actions[i].getValue(Action.NAME) ;
         actionHashtable.put(name, actions[i]) ;
      }
    }

   /**
    * sets an action's icon, tip, 
    */
    public Action setActionProperty(Action a, String toolTip, String icon) {
       String path = imagePath + icon ;
       URL url = ClassLoader.getSystemResource(path);   
       a.putValue(Action.SMALL_ICON, new ImageIcon(url) ) ;
       a.putValue(Action.NAME, toolTip) ;
       a.putValue(Action.SHORT_DESCRIPTION, toolTip) ;
       return a ;
    }
       

   /**
    * Returns list of available fonts on the system.
    */
    public String[] getFontList() {
       GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment()  ;
       String[] fonts = ge.getAvailableFontFamilyNames() ;
       return fonts ;
    }

   /**
    * Returns an instance of SadissToolBar and returns a toolbar
    */
   public JToolBar getToolBar() {
      if (sadissToolBar == null) {         
         sadissToolBar = new SadissToolBar() ;
      }
      
      return sadissToolBar.makeToolBar() ;
   }


   /**
    * Returns an instance of SadissToolBar
    */
   public SadissToolBar getSadissToolBar() {
      if (sadissToolBar == null) {         
         sadissToolBar = new SadissToolBar() ;
      }
      
      return sadissToolBar ;
   }

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * SadissToolBar is an inner class that builds toolbar for Sadiss
    * <!-- ----------------------------------------------------------------------- -->
    */
   class SadissToolBar {
      JToolBar toolbar = new JToolBar() ;
      JButton newButton,   openButton,       saveButton, saveAsButton, 
              printButton, searchButton,     copyButton, cutButton, 
              pasteButton, helpButton, unicodeButton ;         
      JLabel kwkLabel = 
        new JLabel(new ImageIcon(ClassLoader.getSystemResource(imagePath+"kwk.gif") ) ) ;

      /**
       * Returns a saveButton
       */
      JButton getSaveButton() {
         return saveButton ;
      }

      /**
       * Returns kwkLabel
       */
      JLabel getKWKLabel() {
         return kwkLabel ;
      }

      /**
       * Disables selected toolbar buttons when there isn't object to work on
       */
      void setEnabled(boolean state) {
         saveButton.setEnabled(state) ;
         saveAsButton.setEnabled(state) ;
         printButton.setEnabled(state) ;
         searchButton.setEnabled(state) ;
         copyButton.setEnabled(state) ;
         cutButton.setEnabled(state) ;
         pasteButton.setEnabled(state) ;
      }
  
      /**
       * Constructor
       */
      private SadissToolBar() {      
         newButton = toolbar.add( getNewAction() ) ;
         newButton.setRequestFocusEnabled(false) ; 

         openButton = toolbar.add( getOpenAction() ) ;
         openButton.setRequestFocusEnabled(false) ;

         saveButton = toolbar.add( getSaveAction() ) ;
         saveButton.setRequestFocusEnabled(false) ;

         saveAsButton = toolbar.add( getSaveAsAction() ) ;
         saveAsButton.setRequestFocusEnabled(false) ;

         printButton = toolbar.add( getPrintAction() ) ;
         printButton.setRequestFocusEnabled(false) ;
         toolbar.addSeparator() ;

         searchButton = toolbar.add( getSearchAction() ) ;
         searchButton.setRequestFocusEnabled(false) ;
         toolbar.addSeparator( new Dimension(20, 10) ) ;

         copyAction = setActionProperty(
                  (Action) actionHashtable.get(DefaultEditorKit.copyAction),
                  "Copy", "Copy16.gif" ) ;
         copyButton = toolbar.add(copyAction) ;
         copyButton.setRequestFocusEnabled(false) ;

         cutAction = setActionProperty(
                  (Action) actionHashtable.get(DefaultEditorKit.cutAction),
                  "Cut", "Cut16.gif" ) ;
         cutButton = toolbar.add(cutAction) ;
         cutButton.setRequestFocusEnabled(false) ;

         pasteAction = setActionProperty(
                  (Action) actionHashtable.get(DefaultEditorKit.pasteAction),
                  "Paste", "Paste16.gif" ) ;
         pasteButton = toolbar.add(pasteAction) ;
         pasteButton.setRequestFocusEnabled(false) ;

         toolbar.addSeparator() ;         

         selectAllAction = (Action) actionHashtable.get(DefaultEditorKit.selectAllAction) ;

            // reflects the current state of the keyboard mode. if enabled, its KWK; otherwise regular
         Insets inset = saveButton.getInsets() ;
         kwkLabel.setBorder( BorderFactory.createCompoundBorder(
                  BorderFactory.createEtchedBorder(EtchedBorder.RAISED),
                  BorderFactory.createEmptyBorder(inset.top/2, inset.left/2, inset.bottom/2, inset.right/2) ) ) ;
         kwkLabel.setRequestFocusEnabled(false) ;
         toolbar.add(kwkLabel) ;
         toolbar.addSeparator() ;         

            //  Construct and add combobox for font selection
         String[] fontList = getFontList() ;
         JComboBox fontCombo = createComboBox(fontList, 25) ;

            // set combobox size so that it remains fixed on a single size
         fontCombo.setMinimumSize(new Dimension(175, 20)) ;
         fontCombo.setMaximumSize(new Dimension(175, 20)) ;
         fontCombo.setEditable(true) ;
         fontCombo.setSelectedItem("Ethiopia Jiret") ;

         fontCombo.setRequestFocusEnabled(false) ;
         fontCombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
               JComboBox jcb = (JComboBox)e.getSource() ;

               JDesktopPane desktop = SadissJournal.getDesktop() ;
               SadissDocumentFrame sdf = (SadissDocumentFrame)desktop.getSelectedFrame() ;
               if (sdf == null) return ;                     
               JTextPane textPane = sdf.getTextPane() ;

	            int fontSize = StyleConstants.getFontSize(textPane.getCharacterAttributes()); 
               font = new Font((String) jcb.getSelectedItem(), Font.PLAIN, fontSize) ;
               textPane.setFont( font );

               /*
               SimpleAttributeSet sas = new SimpleAttributeSet() ;
	            StyleConstants.setFontFamily(sas, (String) jcb.getSelectedItem() ) ;
               StyleConstants.setFontSize(sas, fontSize) ;
	            textPane.setCharacterAttributes(sas, false) ;   
               */
                  // returns focus to the selected frame (it is not working)
               try {
                  textPane.requestFocus() ;
               } catch (Exception ex) {
                  System.out.println(ex) ;
               }
            }
         } ) ;
         toolbar.add(fontCombo) ;     

            // Construct and add a combobox for fonr size selection
         String[] fontSizeList = {"8","9","10","12","14","16","18","20","22","24",
                                  "26", "28","36","48","72"} ;
         JComboBox fontSizeCombo = createComboBox(fontSizeList, 3) ;
         fontSizeCombo.setSelectedIndex(5) ;

            // set combobox size so that it remains fixed on a single size
         fontSizeCombo.setMinimumSize(new Dimension(50, 20)) ;
         fontSizeCombo.setMaximumSize(new Dimension(50, 20)) ;

         fontSizeCombo.setEditable(true) ;
         fontSizeCombo.setRequestFocusEnabled(false) ;
            
         fontSizeCombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
               JComboBox jcb = (JComboBox)e.getSource() ;

               JDesktopPane desktop = SadissJournal.getDesktop() ;
               SadissDocumentFrame sdf = (SadissDocumentFrame)desktop.getSelectedFrame() ;
               if (sdf == null) return ;                     
               JTextPane textPane = sdf.getTextPane() ;
	            String fontFamily = StyleConstants.getFontFamily(textPane.getCharacterAttributes()); 
               font =  new Font(fontFamily, Font.PLAIN, 
                                Integer.parseInt((String) jcb.getSelectedItem())) ;
               textPane.setFont(font) ;

               /*
	            SimpleAttributeSet sas = new SimpleAttributeSet() ;
               StyleConstants.setFontFamily(sas, fontFamily) ;
	            StyleConstants.setFontSize(sas, Integer.parseInt((String) jcb.getSelectedItem()) ) ;
	            textPane.setCharacterAttributes(sas, false) ;   
               */
                  // returns focus to the selected frame (it is not working)
               try {
                  textPane.requestFocus() ;
               } catch (Exception ex) {
                  System.out.println(ex) ;
               }
            }
         } ) ;
         toolbar.add(fontSizeCombo) ;
         toolbar.addSeparator( new Dimension(20, 10) ) ;

         unicodeButton = toolbar.add( getUnicodeAction() ) ;
         unicodeButton.setRequestFocusEnabled(false) ; 

         helpButton = toolbar.add( getHelpAction() ) ;
      }  

      /**
       * Returns a toolbar 
       */
      public JToolBar makeToolBar() {
         return toolbar ;
      }
   }  // close class

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * SadissDocumentFrame is an inner class. It is a subclass of JInternalFrame.
    * An instance of this class contains a JTextPane object wrapped by 
    * JScrollPane object with Undo/Redo capability. Everytime, an instance is
    * created, the SadissJournal is updated. It has an anonymous class that inherits
    * InternalFrameAdapter in which it implements the internalFrameClosing method.
    * Here it ensures that the document associated with the frame is saved if it is 
    * not.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class SadissDocumentFrame extends JInternalFrame {       
      final int offset  = 0;
      private SadissTextPane textPane = new SadissTextPane() ;
      private String fileName   = "" ;
      private boolean saveState = false ;
      private SadissDocumentFrame sdf ;
      private boolean cancelClose = false ;
      private JMenuItem menuItem ;
      /**
       * Returns the textPane of this SadissDocumentFrame
       */
      SadissTextPane getTextPane() {
         return textPane ;
      }

      /**
       * Sets file name of this document
       * @param fileName a file where this document is saved
       */
      void setFileName(String fileName) {
         this.fileName = fileName ;
      }

      /**
       * Returns the file this document
       */
      String getFileName() {
         return fileName ;
      }

      /**
       * Sets state of file save
       * @param state indicated if the file is saved from last update or not
       */
      void setSaveState(boolean saveState) {
         this.saveState = saveState ;
      }

      /**
       * Returns the save state of this file
       */
      boolean getSaveState() {
         return saveState ;
      }

      /**
       * Constructor intializes titled document
       * @param fileName a file name for a document
       */
      public SadissDocumentFrame(String fileName) {
         super(fileName, true, true, true, true) ;
         setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE) ;
         putClientProperty("JInternalFrame.isPalette", Boolean.TRUE) ;

         setSize(600, 500) ;
         sdf= this ;
         this.fileName = fileName ;

         textPane.setFont( font ) ; 
         JScrollPane jsp = new JScrollPane(textPane, 
                                 ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				                     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS) ;

            // set textPane shortcut key for editing
         Keymap map = textPane.getKeymap() ;
         KeyStroke cut = KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK, false) ;
         map.addActionForKeyStroke(cut, (Action) actionHashtable.get(DefaultEditorKit.cutAction)) ;

         KeyStroke copy = KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK, false) ;
         map.addActionForKeyStroke(copy, (Action) actionHashtable.get(DefaultEditorKit.copyAction)) ;

         KeyStroke paste = KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK, false) ;
         map.addActionForKeyStroke(paste, (Action) actionHashtable.get(DefaultEditorKit.pasteAction)) ;

            // add an undo listener to the current document
         StyledDocument doc = textPane.getStyledDocument() ;
         doc.addUndoableEditListener(new UndoableEditListener() {
            public void undoableEditHappened(UndoableEditEvent e) {
               undoManager.addEdit(e.getEdit()) ;
               undoAction.updateUndoState() ;
               redoAction.updateRedoState() ;
            }
         } ) ;

            // add a listener to monitor insert event into the document
         doc.addDocumentListener(new DocumentListener() {
            public void insertUpdate(DocumentEvent e) {
               setSaveState(false) ;
            }
            public void changedUpdate(DocumentEvent e) {
               setSaveState(false) ;            
            }
            public void removeUpdate(DocumentEvent e) {
               setSaveState(false) ;
            }
         }) ;


            // add the jsp into the internal frame
         getContentPane().add(jsp) ;

            // set its initial location
         setLocation(xDocPos%500, yDocPos%500) ;
         xDocPos += inset ;
         yDocPos += inset ; 

            // add a listener to override a cancled internalframe closing
         addVetoableChangeListener(new VetoableChangeListener() {
            public void vetoableChange(PropertyChangeEvent ev)
                                          throws PropertyVetoException {
               String name = ev.getPropertyName();
               if (name.equals(JInternalFrame.IS_CLOSED_PROPERTY)
                                       && (ev.getNewValue() == Boolean.TRUE)) {
                  if (cancelClose)
                     throw new PropertyVetoException("Exit terminated", ev);
                  else {
                     cancelClose = false ;
                     ((JFrame)owner).setTitle("Sadiss 1.0") ;  // reset the frame title
                  }
               } else if (name.equals(JInternalFrame.IS_SELECTED_PROPERTY)
                                       && (ev.getNewValue() == Boolean.TRUE)) {
                     // set the app frame title 
                  ((JFrame)owner).setTitle("Sadiss 1.0" + "  [" + sdf.getFileName() + "]") ;
                  JDesktopPane desktop = SadissJournal.getDesktop() ;
                  JInternalFrame[] jif = desktop.getAllFrames() ;
               }
            }
         }) ;

            // add a listener and override internalFrameClosing()
         addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameClosing(InternalFrameEvent e) {
               File currentFile = null ;

                  // if the document is zero length, just dispose it
               StyledDocument doc = textPane.getStyledDocument() ;
               if (doc.getLength() == 0) {
                  ((JFrame)owner).setTitle("Sadiss 1.0") ;  // clear titlebar before disposing frame
                  SadissJournal.getDesktop().getDesktopManager().closeFrame(sdf) ; // dispose the frame
                  removeDocumentMenuItem() ;                // remove document menuitem
                  return ;
               }

               if (getSaveState() == false) {

                  int state = sfs.isYesNoCancel(sdf, "File is not saved. Do you want to save it?") ;
                  switch(state) {
                     case 0 : 
                        cancelClose = false ;   // continue exiting
                        sadissToolBar.getSaveButton().doClick() ; break ;
                     case 1 :
                        cancelClose = false ;   // continue exiting
                        break ;
                     case 2 : 
                        cancelClose = true ;    // user cancel closing the application
                        return ; 
                  }
               }
               ((JFrame)owner).setTitle("Sadiss 1.0") ;  // clear titlebar before disposing frame
               SadissJournal.getDesktop().getDesktopManager().closeFrame(sdf) ; // dispose the frame
               removeDocumentMenuItem() ;                // remove document menuitem
            }  // close method
         } ) ;

            // update SadissJournal
         SadissJournal.insertInternalFrame(title, this) ;
         setVisible(true) ;
         sadissToolBar.setEnabled(true) ;
                     
            // create menuitem to be displayed under Window menu
         setDocumentMenuItem(title) ;
      } // close class

      /**
       * Renames the document menuitem to new file name
       */
      void renameDocumentMenuItem(String text) {
         menuItem.setText(text) ;
      }

      /**
       * Removes the document menuitem from the Window menu
       */
      void removeDocumentMenuItem() {
         JMenuBar menuBar = getSadissMenuBar() ;
         menuBar.getMenu(menuBar.getMenuCount() - 2).remove(menuItem) ;
      }
      
      /**
       * Adds a document menuitem to the window menu when a new document is
       * created. The list of current documents in work is kept there.
       * @param title a file name for a document
       */
      void setDocumentMenuItem(String title) {
         menuItem = new JMenuItem(title) ;   
         final JMenuBar menuBar = getSadissMenuBar() ;
         final JMenu window = menuBar.getMenu(menuBar.getMenuCount() - 2) ;
         window.add(menuItem, 0) ; //add to the Window Menu         
         menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                              
               try {
                  sdf.setSelected(true) ;
               } catch (Exception ex) {
                  System.out.println(ex) ;
               }
                  // when a document is selected, reposition its place in the Window menu
               window.remove(menuItem) ;
               window.add(menuItem, 0) ; 
                  // set the app frame title 
               ((JFrame)owner).setTitle("Sadiss 1.0" + "  [" + sdf.getFileName() + "]") ;
            }
         }) ;
      }         
   }

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * NewAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it creates an 
    * instance of SadissDocumentFrame, and adds the frame to the desktop.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class NewAction extends AbstractAction {

      /**
       * Constructor
       */
      NewAction() {  
         super("New...") ;
      }

      public void actionPerformed(ActionEvent e) {
         JDesktopPane desktop = SadissJournal.getDesktop() ;
         internalFrameCount++ ;
         String untitle = "untitle" + internalFrameCount + ".utf" ;
         SadissDocumentFrame sdf = createDocumentFrame(untitle) ;
         desktop.add(sdf, 0) ;

         try {
            sdf.setSelected(true) ;
         } catch (Exception ex) {
            System.out.println(ex) ;
         }
            // set the app frame title 
         ((JFrame)owner).setTitle("Sadiss 1.0" + "  [" + untitle + "]") ;
      }
   }

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * OpenAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it opens a new
    * file, creates an instance SadissDocumentFrame, and adds the frame to the
    * desktop.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class OpenAction extends AbstractAction {
      /**
       * Constructor
       */
      OpenAction() { 
         super() ;
      }

      public void actionPerformed(ActionEvent e) {
         File currentFile = sfs.saveGetFile("Open") ;
         if (currentFile == null) return ;

         SadissDocumentFrame sdf = createDocumentFrame(currentFile.getAbsolutePath()) ;
    	   SadissFileFilter sff = new SadissFileFilter(SadissX, desc) ;
         sfs = SadissFileService.getInstance(sdf, sff) ;

         try {
            if (currentFile != null) {
  	            StringBuffer text = sfs.openSadissFile(currentFile) ;
               JTextPane textPane = sdf.getTextPane() ;
               StyledDocument doc = textPane.getStyledDocument() ;
               AttributeSet as = textPane.getCharacterAttributes() ;            
               doc.insertString(0, text.toString(), as) ;  
               SadissJournal.getDesktop().add(sdf, 0) ;
               sdf.setSaveState(true) ;

                  // make this internal frame active
               try {
                  sdf.setSelected(true) ;
               } catch (Exception ex) {
                  System.out.println(ex) ;
               }
	         }
         } catch (Exception ex) {
            System.out.println("Sadiss Error\n" + e) ;
         }
      }
   }


   /**
    * <!-- ----------------------------------------------------------------------- -->
    * SaveAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it enables
    * the user to save the document into a file.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class SaveAction extends AbstractAction {
      /**
       * Constructor
       */
      SaveAction() { 
         super() ;
      }

      public void actionPerformed(ActionEvent e) {

         JDesktopPane desktop  = SadissJournal.getDesktop() ;
         SadissDocumentFrame sdf = (SadissDocumentFrame) desktop.getSelectedFrame() ;

            // if current document is save, then terminate this method
         if (sdf.getSaveState() == true) 
            return ;

         JTextPane textPane = sdf.getTextPane() ;
         File currentFile = new File(sdf.getFileName()) ;

         if ( sdf.getFileName().startsWith("untitle") ) {
	         currentFile = sfs.saveGetFile("Save") ;     

               // if the file exists, prompt the user what to do about it
            while (currentFile != null && currentFile.exists() && 
                                          !sfs.isYesNo(sdf, "Overwrite existing file?") ) 
               currentFile = sfs.saveGetFile("Save") ;     
         }

         if (currentFile != null) {
            String fileName = currentFile.getAbsolutePath() ;
            sfs.saveSadissFile(currentFile, textPane.getStyledDocument()) ;
            sdf.setTitle(fileName) ;
            sdf.setFileName(fileName) ;
            sdf.setSaveState(true) ;
               // set the app frame title 
            ((JFrame)owner).setTitle("Sadiss 1.0" + "  [" + sdf.getTitle() + "]") ;
         } 

         try {
            sdf.setSelected(true) ;
         } catch (Exception ex) {
            System.out.println(ex) ;
         }

      }  // close method
   }  // close class

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * SaveAsAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it enables
    * the user to save the document into a new or existing file.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class SaveAsAction extends AbstractAction {
      /**
       * Constructor
       */
      SaveAsAction() { 
         super() ;
      }

      public void actionPerformed(ActionEvent e) {
         JDesktopPane desktop  = SadissJournal.getDesktop() ;
         SadissDocumentFrame sdf = (SadissDocumentFrame) desktop.getSelectedFrame() ;

         JTextPane textPane = sdf.getTextPane() ;

         File currentFile = null ;
         currentFile = sfs.saveGetFile("Save As") ;     

            // if the file exists, prompt the user what to do about it
         while (currentFile != null && currentFile.exists() && 
                                             !sfs.isOverWriteAllowed(sdf) ) 
            currentFile = sfs.saveGetFile("Save") ;     

         if (currentFile != null) {
            String fileName = currentFile.getAbsolutePath() ;
            sfs.saveSadissFile(currentFile, textPane.getStyledDocument()) ;
            sdf.setTitle(fileName) ;
            sdf.setFileName(fileName) ;
            sdf.setSaveState(true) ;
            sdf.renameDocumentMenuItem(fileName) ;
               // set the app frame title 
            ((JFrame)owner).setTitle("Sadiss 1.0" + "  [" + sdf.getTitle() + "]") ;
         }
      }
   }

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * CloseAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it allows
    * the selected SadissDocumentFrame to close itself.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class CloseAction extends AbstractAction {
      /**
       * Constructor
       */
      CloseAction() { 
         super() ;
      }

      public void actionPerformed(ActionEvent e) {

         JDesktopPane desktop  = SadissJournal.getDesktop() ;
         JInternalFrame jif = desktop.getSelectedFrame() ;
         try {
            desktop.setSelectedFrame(jif) ;
            jif.setClosed(true) ;
         } catch (Exception ex) {
            System.out.println(ex) ;
         }
      }
   }

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * ExitAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it allows
    * all SadissDocumentFrame to close themselves. It then, terminates the
    * program
    * <!-- ----------------------------------------------------------------------- -->
    */
   class ExitAction extends AbstractAction {
      /**
       * Constructor
       */
      ExitAction() { 
         super("Exit") ;
      }

      public void actionPerformed(ActionEvent e) {

         JDesktopPane desktop  = SadissJournal.getDesktop() ;
         JInternalFrame[] sdf = (JInternalFrame[]) desktop.getAllFrames() ;

         try {
            for (int i=0; i < sdf.length; i++) {
               desktop.setSelectedFrame(sdf[i]) ; 
               sdf[i].setClosed(true) ;
            }

            sdf = (JInternalFrame[]) desktop.getAllFrames() ;
            if (sdf.length == 0)
               System.exit(0) ;
         } catch (Exception ex) {
            System.out.println(ex) ;
         }
      }
   }


   /**
    * <!-- ----------------------------------------------------------------------- -->
    * PrintAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it prints
    * a document.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class PrintAction extends AbstractAction {
      PrintAction() {
         super() ;
	   }
      public void actionPerformed(ActionEvent e) {
         PrintRequestAttributeSet pras = new HashPrintRequestAttributeSet();
         PrinterJob job = PrinterJob.getPrinterJob() ;

         SadissDocumentFrame sdf = 
               (SadissDocumentFrame) SadissJournal.getDesktop().getSelectedFrame() ;
         SadissTextPane stp = sdf.getTextPane() ;
         job.setPrintable(stp) ;

         if (job.printDialog(pras)) {
            try {
               job.print(pras) ;
            } catch (Exception ex) {
               System.out.println(ex) ;
            }
         }         
      }
   }

        
   /**
    * <!-- ----------------------------------------------------------------------- -->
    * UndoAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it reverses
    * the last user editing.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class UndoAction extends AbstractAction {
      UndoAction() {
         super("Undo") ;
         setEnabled(false) ;
      }

      public void actionPerformed(ActionEvent e) {
         try {
            undoManager.undo() ;
         }  catch (CannotUndoException cue) {
            System.out.println("Undo failed : " + cue) ;
            cue.printStackTrace() ;
         }
         updateUndoState() ;
         redoAction.updateRedoState() ;
      }

      void updateUndoState() {
         if (undoManager.canUndo()) {
            setEnabled(true) ;
            putValue(Action.NAME, undoManager.getUndoPresentationName()) ;
         } else {
            setEnabled(false) ;
            putValue(Action.NAME, "Undo") ;
         }
      }
   }
            
   /**
    * <!-- ----------------------------------------------------------------------- -->
    * RedoAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it reverses
    * the last undo action.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class RedoAction extends AbstractAction {
      RedoAction() {
         super("Redo") ;
         setEnabled(false) ;
      }

      public void actionPerformed(ActionEvent e) {
         try {
            undoManager.redo() ;
         }  catch (CannotUndoException cue) {
            System.out.println("Redo failed : " + cue) ;
            cue.printStackTrace() ;
         }
         updateRedoState() ;
         undoAction.updateUndoState() ;
      }

      void updateRedoState() {
         if (undoManager.canRedo()) {
            setEnabled(true) ;
            putValue(Action.NAME, undoManager.getRedoPresentationName()) ;
         } else {
            setEnabled(false) ;
            putValue(Action.NAME, "Redo") ;
         }
      }
   }

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * SearchAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it prompts
    * the user for search and perform the task.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class SearchAction extends AbstractAction {

      SadissTextPane testField = new SadissTextPane() ;
      private Element elm = null ;
      private Position posDoc ;
      private boolean freshSearch = true ;
      private Highlighter highlight ;
      private DefaultHighlighter.DefaultHighlightPainter dhp ;    

      /**
       * Constructor intializes Action
       */
      SearchAction() {
         super() ;
         testField.setFont( font ) ;
      }
      
      /**
       * Searches while keeping current startOffset
       */
      public void executeSearch(String searchKey) {

         JDesktopPane desktop = SadissJournal.getDesktop() ;
         SadissDocumentFrame sdf = (SadissDocumentFrame) desktop.getSelectedFrame() ;
         JTextPane textPane = sdf.getTextPane() ;
	      StyledDocument doc = textPane.getStyledDocument() ;

         try {
            ElementIterator ei = new ElementIterator(doc) ;
            elm = ei.first() ;
	         if (freshSearch) 
               posDoc = doc.createPosition( elm.getStartOffset() ) ;              

            SadissStringMatch ssm ; 

            while (elm != null) {
               System.out.println("... getStartOffset --> " + elm.getStartOffset() ) ;
               System.out.println("... getEndOffset   --> " + elm.getEndOffset() ) ;

                  // get document starting & ending points
               int startOffset = elm.getStartOffset() ;
		         int endOffset = elm.getEndOffset() ;

                  // get the text to perform serach on it
               String text = doc.getText(startOffset, endOffset) ;

                  // calculate startOffset, instanitiate a new searcher object, perform search
               int searchStartOffset = posDoc.getOffset() + startOffset ;
               ssm = new SadissStringMatch( text.substring(searchStartOffset, endOffset),  searchKey ) ;  						
               int location = ssm.SadissSearch() ;

                  // get the next element from the styled document
               elm = ei.next() ;

                  // if search successful, position cursor, update position, and break loop
  	            if ( location >= 0) {
                  textPane.requestFocus() ;
                  int newPos = posDoc.getOffset() + location ;

                     // highlight found string
                  highlight = textPane.getHighlighter() ;
                  dhp = new DefaultHighlighter.DefaultHighlightPainter(Color.red)  ;
                  highlight.addHighlight( newPos, newPos + searchKey.length(), dhp ) ;

                     // reposition caret to newly found string, break loop
                  textPane.setCaretPosition( newPos ) ;  
		            posDoc = doc.createPosition( newPos + searchKey.length() ) ;
                  break ;
		         } else {
                  JOptionPane.showMessageDialog( (JFrame) owner, "Search String Not Found!") ;
                  break ; 
               }
            } 
         } catch (Exception msg) {
            System.out.println("executeSearch: " + msg) ; 
         }
      }  // close method


      /**
       * Dialogbox that controls the search process 
       */
      public void showSadissSearchDialog() {
         final JDialog searchDialog = new JDialog((JFrame)owner, "Search", false) ;
         searchDialog.setSize(350, 150) ;

         JLabel searchLabel = new JLabel("Search For") ;
         JButton searchNext = new JButton("Search") ;
         JButton newSearch  = new JButton("New") ;
         JButton cancel     = new JButton("Cancel") ;

         JPanel searchPane = new JPanel() ;
         searchPane.setLayout(new GridLayout(2, 2)) ;
      	searchPane.setBorder(new EmptyBorder(13, 13, 13, 13) ) ;
         searchPane.add(searchLabel) ;
         testField.setMinimumSize(new Dimension(175, 20)) ;
         testField.setFont( font ) ;
         testField.setPreferredSize(new Dimension(200, 20)) ;
         testField.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED)) ;
         searchPane.add(testField) ;


         JPanel buttonPane = new JPanel() ;
	      buttonPane.setBorder(new EmptyBorder(5, 5, 5, 5) ) ;
         buttonPane.add(searchNext) ;
         buttonPane.add(newSearch) ;
         buttonPane.add(cancel) ; 

            // bind action listener to searchNext Button
         searchNext.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ea) {
               String key = testField.getText() ;
               if (key == null) return ;
	               executeSearch(key) ;
		         freshSearch = false ;
            }
	      } ) ;

            // bind action listener to perform a new search using entered key
         newSearch.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ea) {
               String key = testField.getText() ;
               if (key == null) return ;
		            freshSearch = true ;
	            executeSearch(key) ;
            }
	      } ) ;

            // bind action listener to searchNext Button
         cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ea) {
   	         searchDialog.setVisible(false) ;
               searchDialog.dispose() ;
		         freshSearch = true ;

                  // if there exist an instance of highlight, remove all highlights
               if (highlight != null)
                  highlight.removeAllHighlights() ;
            }
	      } ) ;
       
         searchDialog.getContentPane().add(searchPane, BorderLayout.NORTH) ;
         searchDialog.getContentPane().add(buttonPane, BorderLayout.SOUTH) ;

         searchDialog.setLocationRelativeTo( (JFrame) owner ) ;
         searchDialog.setVisible(true) ;
      }  // close method


      /**
       * This actionPerformed method implements the ActionListener from AbstractAction 
       */
      public void actionPerformed(ActionEvent e) {
         showSadissSearchDialog() ;
      }

   } // close Search class

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * HelpAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it creates an 
    * instance of SadissDocumentFrame, and adds the frame to the desktop.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class HelpAction extends AbstractAction {

      HelpAction() {  
         super() ;
      }

      public void actionPerformed(ActionEvent e) {
         JDesktopPane desktop = SadissJournal.getDesktop() ;
         SadissHelp sadissHelp = new SadissHelp() ;
         desktop.add(sadissHelp, 0) ;

            // make this internal frame active
         try {
            sadissHelp.setSelected(true) ;
         } catch (Exception ex) {
            System.out.println(ex) ;
         }
      }
   }  // close HelpAction class

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * ThemeAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it implements
    * the selected theme.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class ThemeAction extends AbstractAction {
      DefaultMetalTheme[] metalTheme = {
                       new KhakiMetalTheme(), new SteelblueMetalTheme(), 
                       new FeldsparMetalTheme(), new LightCharcoalMetalTheme(), 
                       new HighContrastMetalTheme(), new LowContrastMetalTheme(),
                       new DefaultMetalTheme() } ;
      /**
       * Constructor
       */
      ThemeAction() {  
         super() ;
      }

      public void actionPerformed(ActionEvent e) {
         String index = (String) e.getActionCommand() ;
         MetalTheme selectedTheme = metalTheme[ Integer.parseInt(index) ] ;
         MetalLookAndFeel.setCurrentTheme(selectedTheme) ;
         try {
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
         } catch (Exception ex) {
            ex.printStackTrace() ;
         }
	        SwingUtilities.updateComponentTreeUI( (JFrame)owner );
      }
   }  // close ThemeAction class

   /**
    * <!-- ----------------------------------------------------------------------- -->
    * UnicodeAction is an inner class and is a subclass of AbstractAction.
    * It overrides the <b>actionPerformed(..)<.b> methode where it prompts
    * the user for Unicode character entry using code points.
    * <!-- ----------------------------------------------------------------------- -->
    */
   class UnicodeAction extends AbstractAction {
      JTextPane textField = new JTextPane() ;       

      /**
       * Constructor intializes Action
       */
      UnicodeAction() { 
         super() ;
         textField.setFont( font ) ;
      }
      

      /**
       * Dialogbox that controls the search process 
       */
      public void showUnicodeDialog() {
         final JDialog unicodeDialog = new JDialog((JFrame)owner, "Unicode", false) ;
         unicodeDialog.setSize(400, 250) ;

         JLabel label       = new JLabel("Unicode textfield for code points") ;
         JButton enter      = new JButton("Enter") ;
         JButton finish     = new JButton("Finish") ;

         JPanel unicodePane = new JPanel() ;
         unicodePane.setLayout(new GridLayout(2, 2)) ;
      	unicodePane.setBorder(new EmptyBorder(13, 13, 13, 13) ) ;

         unicodePane.add(label) ;
         textField.setMinimumSize(new Dimension(175, 20)) ;
         textField.setFont( font ) ;
         textField.setPreferredSize(new Dimension(288, 64)) ;
         textField.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED)) ;
         unicodePane.add(textField) ;


         JPanel buttonPane = new JPanel() ;
	      buttonPane.setBorder(new EmptyBorder(5, 5, 5, 5) ) ;
         buttonPane.add(enter) ;
         buttonPane.add(finish) ;

            // bind action listener to searchNext Button
         enter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ea) {

                  // obtain the text from th text field
               StringBuffer buffer = new StringBuffer(textField.getText()) ;
   
                  // converts lowercase letters to uppercase for simplicity
               for (int i=0; i < buffer.length(); i++) {
                  char c = buffer.charAt(i) ;
                  if ( Character.isLetter(c) ) {
                     if ( c >= 'a' && c <= 'z')
                        buffer.replace(i, (i+1), ""+ Character.toUpperCase(c)) ;
                  }
               }

                  // get the focussed internal frame.
               JDesktopPane desktop = SadissJournal.getDesktop() ;
               SadissDocumentFrame sdf = (SadissDocumentFrame) desktop.getSelectedFrame() ;  

                  // convert the text which is in sequence of \\uXXXX form to Unicode character
               try {	                        
                  String digits = "0123456789ABCDEF" ;
                  StringBuffer text = new StringBuffer() ;
                  String word = "" ;
                  int code = 0 ;
                  int i = 0 ;
                  i = buffer.indexOf("\\U", i) ;
                  while (i >= 0) {

                     for (int j=i+2; (j < buffer.length()) && (buffer.charAt(j) != '\\') ; j++)
                        word += buffer.charAt(j) ;

                     for (int j=word.length() - 1, pos=0; j >= 0; j--, pos++) {
                        int digit = digits.indexOf(word.charAt(pos)) ;
                        code += digit * (int)(Math.pow(16, j)) ;
                     }
                     text.append((char) code) ;
                     i = buffer.indexOf("\\U", i+2) ;     // get the next unicode character         
                     word = "" ;
                     code = 0 ;
                  }

                  sdf.getTextPane().getStyledDocument().insertString(
                           sdf.getTextPane().getCaretPosition(),text.toString(), null) ;
               } catch (Exception ex) {
                  ex.printStackTrace() ;
               }
            }
	      } ) ;

            // bind action listener to searchNext Button
         finish.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ea) {
   	         unicodeDialog.setVisible(false) ;
               unicodeDialog.dispose() ;               
            }
	      } ) ;
       
         unicodeDialog.getContentPane().add(unicodePane, BorderLayout.NORTH) ;
         unicodeDialog.getContentPane().add(buttonPane, BorderLayout.SOUTH) ;

         unicodeDialog.setLocationRelativeTo( (JFrame) owner ) ;
         unicodeDialog.setVisible(true) ;
      }  // close method


      /**
       * This actionPerformed method implements the ActionListener from AbstractAction 
       */
      public void actionPerformed(ActionEvent e) {
         showUnicodeDialog() ;
      }

   } // close Search class

} // close Sadiss class
