/*
 * @(#)SadissFileFilter.java 1.0 10/24/2002
 * Copyright 2002 Senamirmir Project
 *
 * You may distribute and/or modify Sadiss under the conditions 
 * of the LaTeX Project Public License. The license is located at
 * http://www.latex-project.org/lppl.txt
 */

package keffa ;

import javax.swing.filechooser.*;
import java.io.File;

/**
 * <!-- ----------------------------------------------------------------------- -->
 * SadissFileFilter a subclass of FileFilter which filters Sadiss' files
 * @author abass alamene
 * @version 10/24/2002
 * <!-- ----------------------------------------------------------------------- -->
 */

public class SadissFileFilter extends FileFilter {
   String[] extensions;
   String description;

   /**
    * Constructor
    */
   public SadissFileFilter(String ext) {
      this (new String[] {ext}, null);
   }
   
   /**
    * Constructor
    */
   public SadissFileFilter(String[] exts, String descr) {
      // clone and lowercase the extensions
      extensions = new String[exts.length];
      for (int i = exts.length - 1; i >= 0; i--)  {
         extensions[i] = exts[i].toLowerCase();
      }

      // make sure we have a valid (if simplistic) description
      description = (descr == null ? exts[0] + " files" : descr);
   }

   /**
    * Returns true if the given file is acceptable; otherwise, false.
    * @param f a file object
    */
   public boolean accept(File f) {
     // we always allow directories, regardless of their extension
     if (f.isDirectory()) { return true; }

     // ok, it's a regular file so check the extension
     String name = f.getName().toLowerCase();
     for (int i = extensions.length - 1; i >= 0; i--) {
        if (name.endsWith(extensions[i])) {
           return true;
        }
     }
     return false;
   }

   /**
    * Returns a description of acceptable file types based on their extensions.
    */
   public String getDescription() { return description; }
}
