/*
 * @(#)SadissComboBoxEditor.java 1.0 10/24/2002
 * Copyright 2002 Senamirmir Project
 *
 * You may distribute and/or modify Sadiss under the conditions 
 * of the LaTeX Project Public License. The license is located at
 * http://www.latex-project.org/lppl.txt
 */

package keffa ;

import java.io.* ;
import java.awt.* ;
import javax.swing.* ;
import javax.swing.text.* ;
import java.awt.event.* ;
import java.util.* ;
import java.net.URL ;

/**
 * <!-- ----------------------------------------------------------------------- -->
 * SadissJournal is an inventory class that keeps track of documents and their
 * properties. It implements HashMap. All fields and methods are static.
 * @author abass alamene
 * @version 1.0 10/24/2002
 * <!-- ----------------------------------------------------------------------- -->
 */

class SadissJournal {
   private static HashMap internalFrame = new HashMap() ;
   private static HashMap fileName      = new HashMap() ;
   private static HashMap textPane      = new HashMap() ;
   private static HashMap saveState     = new HashMap() ;
   private static JDesktopPane desktop  = new JDesktopPane() ;
   private static Locale currentLocale;
   private static ResourceBundle currentResourceBundle;
   static Font defaultFont ;
   private static SadissControl sc ;

   /**
    * Sets the current SadissControl
    * @param newSadissControl a new instance of SadissControl
    */
   static void setSadissControl(SadissControl newSadissControl) {
      sc = newSadissControl ;
   }
   
   /**
    * Returns the current SadissControl
    */
   static SadissControl getSadissControl() {
      return sc ;
   }


   /**
    * Returns an instance of a Font class
    */
   public static Font getDefaultFont() {
         return defaultFont ;
   }

   /**
    * Returns user created truetype font
    * @param fontFileName the font file from which a new font is created
    * @param size font size
    */
   public static Font createFont(String fontFileName, float size) {
      if (defaultFont != null) return defaultFont ;
      try {
         URL fontURL = new SadissJournal().getClass().getResource(fontFileName);
         InputStream fontStream = fontURL.openStream();
         defaultFont = Font.createFont(Font.TRUETYPE_FONT,fontStream);
         defaultFont = defaultFont.deriveFont(size) ;
         fontStream.close();
      } catch (Exception ex) {
         ex.printStackTrace();
      }
      return defaultFont ;
   }

   /**
    * Sets the default font to newFont
    * @param faceName name of a font
    * @param style font style: italic, plain, bold
    * @param size font size
    */
   public static void setFont(String faceName, int style, int size) {
      defaultFont = new Font(faceName, Font.PLAIN, size) ;
   }

   /**
    * Sets the current Locale
    * @param newLocale a new instance of Locale
    */
   static void setLocale(Locale newLocale) {
      currentLocale = newLocale ;
   }
   
   /**
    * Returns the current Locale
    */
   static Locale getLocale() {
      return currentLocale ;
   }
   

   /**
    * Sets the current ResourceBundle
    * @param newResourceBundle a new ResourceBundle instance
    */
   static void setResourceBundle(ResourceBundle newResourceBundle) {
      currentResourceBundle = newResourceBundle ;
   }
   
   /**
    * Returns the current Locale
    */
   static ResourceBundle getResourceBundle() {
      return currentResourceBundle ;
   }

   /**
    * Sets the current JDesktopPane
    * @param desktop A single instance of Sadiss JDesktopPane object
    */
   static void setDesktop(JDesktopPane newDesktop) { 
      desktop = newDesktop ; 
   }

   /**
    * Gets the current JDesktopPane object
    */
   static JDesktopPane getDesktop() { 
      return desktop ; 
   }

   /**
    * Inserts a JInternalFrame object with its title as key
    * @param key the title of the internal frame which is used as a key
    * @param newInternalFrame a new document internal frame
    */
   static void insertInternalFrame(String key, JInternalFrame newInternalFrame) { 
      internalFrame.put(key, newInternalFrame) ;
   }

   /**
    * Removes a JInternalFrame object from the HashMap
    * @param key the title of the internal frame which is used as a key
    */
   static void removeInternalFrame(String key) {
      internalFrame.remove(key) ;
   }

   /**
    * Returs the requested JInternalFrame object
    * @param key the title of the internal frame which is used as a key
    */
   static JInternalFrame getInternalFrame(String title) { 
      return (JInternalFrame) internalFrame.get(title) ; 
   }

   /**
    * Inserts the file name of this document
    * @param key the title of the internal frame which is used as a key
    * @param newFileName a new document file name
    */
   static void insertFileName(String key, String newFileName) { 
      fileName.put(key, newFileName) ;
   }

   /**
    * Removes the file name of this document from HashMap
    * @param key the title of the internal frame which is used as a key
    */
   static void removeFileName(String key) {
      fileName.remove(key) ;
   }

   /**
    * Returns the requested file name that matches the key
    * @param key the title of the internal frame which is used as a key
    */
   static String getFileName(String title) { 
      return (String) fileName.get(title) ; 
   }

   /**
    * Inserts the new document JTextPane object into a hashmap
    * @param key the title of the internal frame which is used as a key
    * @param newTextPane the new document JTextPane object
    */
   static void insertTextPane(String key, JTextPane newTextPane) { 
      textPane.put(key, newTextPane) ;
   }

   /**
    * Removes a document JTextPane object from the hashmap
    * @param key the title of the internal frame which is used as a key
    */
   static void removeTextPane(String key) {
      textPane.remove(key) ;
   }

   /**
    * Returs the JTextPane object that matches the key
    * @param key the title of the internal frame which is used as a key
    */
   static JTextPane getTextPane(String title) { 
      return (JTextPane) textPane.get(title) ; 
   }

   /**
    * Inserts the state of the current document if it is saved or not
    * @param key the title of the internal frame which is used as a key
    * @param newSaveState a boolean value indicates if the file is saved or not
    */
   static void insertSaveState(String key, boolean newSaveState) { 
      Boolean state = new Boolean(newSaveState) ;
      saveState.put(key, state) ;
   }

   /**
    * Removes the state of save for this document
    * @param key the title of the internal frame which is used as a key
    */
   static void removeSaveState(String key) {
      saveState.remove(key) ;
   }

   /**
    * Returns the state of a file save
    * @param key the title of the internal frame which is used as a key
    */
   static boolean getSaveState(String title) { 
      return ((Boolean) saveState.get(title)).booleanValue() ; 
   }
}
