/*
 * @(#)SadissComboBoxEditor.java 1.0 10/24/2002
 * Copyright 2002 Senamirmir Project
 *
 * You may distribute and/or modify Sadiss under the conditions 
 * of the LaTeX Project Public License. The license is located at
 * http://www.latex-project.org/lppl.txt
 */

package keffa ;

import java.awt.* ;
import javax.swing.* ;
import javax.swing.text.* ;
import java.awt.print.*;

/**
 * <!-- ----------------------------------------------------------------------- -->
 * SadissTextPane is a subclass of JTextPane. It adds antialias.
 * @author abass alamnehe
 * @version 1.0 10/24/2002
 * <!-- ----------------------------------------------------------------------- -->
 */

class SadissTextPane extends JTextPane implements Printable {

   /**
    * Constructor
    */
   SadissTextPane() {      

         // set tabs
      final int tabCount = 28 ;
      TabStop[]  tabStop = new TabStop[tabCount] ;
      for (int i=1; i <= tabCount; i++) {
         float f = (float) (i * 27);
         tabStop[i-1]= new TabStop(f, TabStop.ALIGN_LEFT, TabStop.LEAD_NONE) ;
      }

      TabSet tab = new TabSet(tabStop) ;
      SimpleAttributeSet sas = new SimpleAttributeSet() ;
      StyleConstants.setTabSet(sas, tab) ;
      getStyledDocument().setParagraphAttributes(0, 1, sas, false) ;      

         // add Ethiopic (KWK) keyboard layout
      addKeyListener( new SadissKeyboard(this, getStyledDocument()) ) ;
      setPreferredSize(new Dimension(2000, 10000)) ;  // prevent linewrap
   }

/**
 * Implements this method from JEditorPane. The purpose is to prevent
 * auto linewrap as much as possible.
 */
public boolean getScrollableTracksViewportWidth() {
   return false ;
}

   /**
    * Implements antialias
    * @param g device context
    */
   public void paintComponent(Graphics g) {
      Graphics2D g2 = (Graphics2D) g;
      g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
      super.paintComponent(g2);
   }

   /**
    * Implements the print method of Printable
    * @param g device context
    * @param pf page info
    * @param pageIndex page number requested for printing
    */
   public int print(Graphics g, PageFormat pf, int pageIndex) throws PrinterException {
      Graphics2D g2 = (Graphics2D)g;
      g2.setColor(Color.black);

         // set default foreground color to black
      //for faster printing, turn off double buffering
      //RepaintManager.currentManager(this).setDoubleBufferingEnabled(false);

      Dimension d = this.getSize();       // get size of document
      double panelWidth = d.width;        // width in pixels
      double panelHeight = d.height;      // height in pixels

      double pageHeight = pf.getImageableHeight(); // height of printer page
      double pageWidth = pf.getImageableWidth();   // width of printer page

      double scale = pageWidth/panelWidth;
      int totalNumPages = (int)Math.ceil(scale * panelHeight / pageHeight);

         // make sure not print empty pages
      if(pageIndex >= totalNumPages) { 
         return Printable.NO_SUCH_PAGE;
      }

         // shift Graphic to line up with beginning of print-imageable region
      g2.translate(pf.getImageableX(), pf.getImageableY());

         // shift Graphic to line up with beginning of next page to print
      g2.translate(0f, -pageIndex*pageHeight);

         // scale the page so the width fits...
      g2.scale(scale, scale);

      this.paint(g2); //repaint the page for printing

      return Printable.PAGE_EXISTS;
   }
}
